st7/
;******************************************************************************
; LITEcomp - aplikacje
; Biblioteka procedur obsugi magistrali 1-wire
;******************************************************************************
		.NOLIST	
		#INCLUDE "st7flite19.inc"
    .LIST
		
		#define OWDIR	PBDDR
		#define	OWDAT PBDR

DQ  EQU 0
;******************************************************************************
; obszar pamici danych RAM
;******************************************************************************
		BYTES
		segment	'ram0'
owdata   	DS.B  1

;******************************************************************************
; obszar pamieci programu		
;******************************************************************************
		WORDS	
		segment 'rom'; poczatek segmentu pamieci programu
;******************************************************************************
; Procedura opniajca o ok. A + 2,5 us dla fosc=16MHz (fcpu=8MHz)
;******************************************************************************
.owdelay
		NOP
		DEC  	A
		JRNE 	owdelay
		RET

;******************************************************************************
; Procedura oczekiwania na zakoczenie pomiaru temperatury
;******************************************************************************
.waitEndConversion
		CALL	owreadbit					; odczyt slotu
		JRNC	waitEndConversion	; jeli zero to skok na pocztek
		RET

;******************************************************************************
; procedura generujca impuls reset 
; i sprawdzajca impuls obecnoci
; ukadu 1-wire
;******************************************************************************
.owreset
	  BSET 	OWDIR, #DQ 				; wymuszamy stan niski na linii DQ
		LD 		A, #240 					;
		CALL 	owdelay 					; | czekamy ok 480us
 		LD 		A, #240 					; |
		CALL 	owdelay 					;
		BRES 	OWDIR, #DQ 				; zwalniamy lini DQ
		
		LD 		A, #110 					; 
		CALL 	owdelay						; czekamy ok 110us

		SCF 										; ustawiamy wskanik C
		BTJT 	OWDAT, #DQ, owr1 	; jel linia DQ jest w stanie wysokim 
														; (brak impuslu obecnoi) to skaczemy do owr1
		RCF 										; w przeciwnym razie zerujemy wskanik C
owr1
		LD 		A, #180 					; 
		LD 		A, #180 					; |
		CALL 	owdelay						;
		RET

;*******************************************************************************
; procedura zapisu bitu
; bit do zapisania przekazywany jest przez wskanik C
;*******************************************************************************
.owwritebit
    BSET 	OWDIR, #DQ 				; wymuszamy na linii DQ stan niski

		LD 		A, #13  					;
		CALL 	owdelay						; czekamy ok 15us
		JRNC 	owb1 							; jesli wskanik C jest wyzerowany (zapis zera)
														; to skaczemy do owb1
		BRES 	OWDIR, #DQ 				; w przeciwnym razie zwalniamy lini DQ

.owb1		
		LD 		A, #100 					; 
		CALL 	owdelay						; czekamy ok 100us
		BRES 	OWDIR, #DQ 				; zwalniamy lini DQ
		
		LD 		A, #100 					; 
		CALL 	owdelay						; czekamy do koca slotu
		RET

;********************************************************************************
; procedura odczytu bitu
; odczytany bit przekazywany jest przez wskanik C
;********************************************************************************
.owreadbit
		BSET 	OWDIR, #DQ 				; wymuszamy na linii DQ stan niski
		BRES 	OWDIR, #DQ 				; zwalniamy lini DQ
		LD 		A, #10  					;
		CALL 	owdelay 					; czekamy ok 12us 
		SCF 										; ustawiamy wskanik przeniesienia
		BTJT 	OWDAT, #DQ, owrb1	; jeli linia DQ jest w stanie wysokim
														; to SLAVE nadaje "1"
		RCF 										; w przeciwnym razie nadaje "0" i zerujemy wsk. C
.owrb1
    LD 		A, #100 					;
		CALL 	owdelay 					; czekamy do koca slotu	
		RET
		
;******************************************************************************
; procedura zapisu bajtu
; bajt do zapisania przekazywany jest przez rejestr A
;******************************************************************************
.owwrite
		PUSH	X
		LD 		owdata, A 			; zapamitujemy dan do zapisania w owdata
		LD 		X, #8 					; ilo powtrze ptli
.ow1
		LD 		A, owdata 			; przywracamy dan do zapisania
    RRC 	A 							; obrt akumulatora w prawo (LS bit do C)
		LD 		owdata, A 			; zapamitujemy obrcony A w owdata
		CALL 	owwritebit 			; wysyamy bit na magistral
		DEC 	X 							; zmniejszamy licznik ptli
		JRNE 	ow1 						; jesli wikszy od zera skaczemy na pocztek ptli
		POP		X
		RET
		
;******************************************************************************		
; procedura odczytu bajtu
; odczytany bajt przekazywany jest przez rejestr A
;******************************************************************************
.owread
		PUSH	X
		LD 		A, #0 					; zerujemy rejestr A i zmien owdata
		LD 		owdata, A
		LD 		X, #8 					; iloc powtrze ptli
.or1
		CALL 	owreadbit 			; orczutyjemy jeden bit
		LD 		A, owdata 			; owdata do A
		RRC 	A								; obrt A w prawo (C na pozycj MS bitu)
		LD 		owdata, A 			; a do owdata
    DEC 	X 							; zmniejszamy licznik ptlo
		JRNE 	or1 						; jesli niezerowy to skok na poczatek ptli
		POP		X
		RET
;******************************************************************************
; Koniec pliku 1wire.asm
;******************************************************************************
		END
		