st7/
;******************************************************************************
; LITEcomp - aplikacje
; Przykad wykorzystania biblioteki procedur obsugi magistrali 1wire
; Zamek z ukadami iButton
;******************************************************************************
		.NOLIST	
		#INCLUDE 	"st7flite19.inc"
		#INCLUDE 	"1wire.inc"
		#INCLUDE	"iButton.inc"
    .LIST
		
LEDRED		EQU 0
LEDGREEN	EQU	1
OUT				EQu	2
BUZZER		EQU	3

NUMOFCODES EQU	8

;******************************************************************************
; obszar pamici danych RAM
;******************************************************************************
		BYTES 					
		SEGMENT 'ram0'	


;******************************************************************************
; obszar pamieci danych EEPROM	
;******************************************************************************
		WORDS 					
		SEGMENT	'eeprom'
buttonCode	DS.B	8*NUMOFCODES

;******************************************************************************
; obszar pamieci programu		
;******************************************************************************		
		SEGMENT 'rom' 	
;******************************************************************************
; Pocztek programu gwnego
;******************************************************************************
._reset
		CALL	Init

		BTJT	PBDR, #6, mainloop
		CALL	registerCodes
		
		
mainloop
		LD		A, #buttonCode.h		;  
		LD		{ptrRead+0}, A			; 
		LD		A, #buttonCode.l		; Inicjalizacja wskanika odczytu
		LD		{ptrRead+1}, A			;

		CALL	readnumer						; odczyt numeru przyoonego do czujnika ukadu
		CALL	crc8								; obliczenie sumy kontrolnej
		LD		A, numer						; zaadowanie do A FAMILY CODE
		JREQ	mainloop						; jeli FAMILY CODE = 0 to skok na pocatek
		LD		A, crc							; zaadowanie do A sumy kontrolnej
		JRNE	mainloop						; jeli CRC != 0 skok na pocztek ptli
		CALL	compareCodes				; w przeciwnym razie sprawdzenie czy kod zarejestrowany
		JRA		mainloop						; skok na pocztek ptli gwnej

;******************************************************************************
; Procedura inicjalizacji portw i zmiennych
;******************************************************************************
Init
		LD		A, #%00001000
		LD		PADR, A
		LD		A, #$FF
		LD		PADDR, A
		LD		PAOR, A
		
		LD		A, #0
		LD		PBDR, A
		LD		PBDDR, A
		LD		A, #%01000000
		LD		PBOR, A
		
		LD		A, #buttonCode.h
		LD		{ptrWrite+0}, A
		LD		A, #buttonCode.l
		LD		{ptrWrite+1}, A
		
		LD		A, #numer.h
		LD		{numptr+0}, A
		LD		A, #numer.l
		LD		{numptr+1}, A
		RET

;******************************************************************************
; Procedura wczajca czerwon diod LED w czytniku na 4s
;******************************************************************************
RedLED
		BSET	PADR, #LEDRED
		CALL	waitTimer
		BRES	PADR, #LEDRED
		RET
		
;******************************************************************************
; Procedura wczajca zielon diod LED w czytniku na 4s
;******************************************************************************		
GreenLED
		BSET	PADR, #LEDGREEN
		CALL	waitTimer
		BRES	PADR, #LEDGREEN
		RET
		
;******************************************************************************
; Procedura ukatywniajaca wyjcie sterujace elektrozamkiem na 4s
;******************************************************************************				
EnableOut
		BSET	PADR, #OUT
		BSET	PADR, #LEDGREEN
		CALL	waitTimer
		BRES	PADR, #LEDGREEN
		BRES	PADR, #OUT
		RET
		
;******************************************************************************
; Procedura wczajaca na chwilk buzzer
;******************************************************************************
Beep
		BRES	PADR, #BUZZER
		CALL	Delay
		BSET	PADR, #BUZZER
		CALL	Delay
		RET

;******************************************************************************
; Procedura opniajca
;******************************************************************************
Delay
		PUSH	A
		PUSH	X
		LD		X, #255
DelayLoop2
		LD		A, #255
DelayLoop3		
		DEC		A
		JRNE	DelayLoop3
		DEC		X
		JRNE	DelayLoop2
		POP		X
		POP		A
		RET

;******************************************************************************
; Procedura zwikszajca wskanik odczytu o 8
;******************************************************************************
.incrReadPtr
		LD		A,{ptrRead+1}
		ADD		A, #8
		LD		{ptrRead+1},A
		RET
		
;******************************************************************************
; Procedura zwikszajca wskanik zapisu o 8
;******************************************************************************
.incrWritePtr
		LD		A,{ptrWrite+1}
		ADD		A, #8
		LD		{ptrWrite+1},A
		RET

;******************************************************************************
; Procedura oczekiwania ok 4 sekund
;******************************************************************************
waitTimer
		LD		A, #%00001000 			; wczenie timera F=1KHz
		LD		ATCSR, A
		BTJF	ATCSR, #2, *				; oczekiwanie na przepenienie
		CLR		ATCSR								; zatrzymanie timera
		RET
		
;******************************************************************************
; Procedura zapisujca kod do pamici EEPROM
;******************************************************************************
.regCode		
		CALL	registerCode				; zarejestrowanie kodu
		CALL	Beep								; piknicie buzzerem
		CALL	GreenLED						; bysnicie zielon diod czytnika
		RET
	

;******************************************************************************
; Procedura rejestrujca kody ukadw DS18B20
;******************************************************************************
.registerCodes
		PUSH	X										; zapamitanie X na stosie
		CLR		X										; wyzerowanie licznika ptli
registerLoop		
		CALL	regCode							; zarejestrowanie jednego kodu
		CALL	incrWritePtr				; zwikszenie wskanika zapisu o 8
		INC		X										; inkrementacja licznika ptli
		CP		X, #NUMOFCODES			; sprawdzenie czy zapisano ju wszystkie kody
		JRNE	registerLoop				; jeli nie, to skok na pocztek ptli
		POP		X
		RET	
		
;******************************************************************************
; Procedura porwnujca odczytany kod z zapisanym w pamici EEPROM
;******************************************************************************
.compareCode
		PUSH	X										; zapamitanie X na stosie
		CLR		X										; wyzerowanie licznika ptli
compareLoop
		LD		A, ([ptrRead.w],X)	; zaadowanie do A bajtu odczytanego kodu
		CP		A, ([numptr.w],X)		; porwnanie z bajtem zapamieanym w pamieci 
		JRNE	badCode							; jeli rne to skok do badCode
		INC		X										; zwikszenie licznika petli
		CP		X, #8								; sprawdzenie, 
		JRNE	compareLoop					; jeli nie to skok na pocztek ptli
		LD		A, #$55							; zaadowanie wartoci 0x55 (dobry kod)
		POP		X										; odtworzenie zawartosi X ze stosu
		RET
badCode
		LD		A, #$AA							; zaadowanie do A wartosci 0xAA (zy kod)
		POP		X										; odtworzenie zawartoci X ze stosu
		RET

;******************************************************************************
; Procedura porwnujca odczytany kod z ca tablic kodw w pamici EEPROM
;******************************************************************************
compareCodes
		PUSH	X										; zapamitanie X na stosie
		CLR		X										; wyzerowanie licznika ptli		
compareLoop1		
		CALL	compareCode					; sprawdzenie kodu
		CP		A, #$55							; sprawdzenie czy odebrany kod jest OK
		JREQ	codeIsOK						; jeli tak to skok do codeIsOK
		CALL	incrReadPtr					; w przeciwnym razie zwikszamy wskanik
		INC		X										; oraz licznik ptli
		CP		X, #NUMOFCODES			; czy sprawdzono wszystkie kody?
		JRNE	compareLoop1				; jeli nie to skok na pocztek ptli		
		
		CALL	Beep								; * numer ukadu iButton niezgodny *
		CALL	Beep								; dwa pikniecia buzzerem
		CALL	RedLED							; waczenie czerwonej diody LED
		POP		X										; odtwarzamy zawarto X ze stosu
		RET
codeIsOK											; * numer ukadu iButton zgodny *
		CALL	Beep								; jedno piknicie buzzera
		CALL	EnableOut						; i aktywacja wyjcia sterujcego ryglem
		POP		X										; odtwarzamy zawarto X ze stosu
		RET
		
;******************************************************************************
; Koniec pliku main.asm
;******************************************************************************
		END
