st7/
;**************************************************************************************************
; LITEcomp - aplikacje
; Termometr z ukadem DS18B20
;**************************************************************************************************
    .nolist	
		#INCLUDE "st7flite19.inc"
		#INCLUDE "1wire.inc"
		#INCLUDE "LCD.inc"
		#include "writedecimal.inc"
    .list       
;******************************************************************************
; obszar pamici danych RAM
;******************************************************************************		
		BYTES
		segment	'ram0'
; zmienne dla DS18B20
tmplsb		DS.B	1
tmpmsb		DS.B	1

; zmienne przechowujace warto temperatury
mintemp		DS.B 1
maxtemp		DS.B 1
temper		DS.B 1

;******************************************************************************
; obszar pamieci programu		
;******************************************************************************
		WORDS	
		segment 'rom'
stopien		DC.B	6,9,9,6,32,32,32,32,0
txt				DC.B 	"    LITEcomp", 0
txt1			DC.B  "  ST7 & DS18B20"
txt2			DC.B	"     T:",0
txt3			DC.B  "MIN:",0
txt4			DC.B	8,"  MAX:",0

;**************************************************************************************************
; poczatek programu gwnego
;**************************************************************************************************
._reset
    LD A, #%11111010	; 	
    LD PADDR, A ;
    		
    LD A, #%11111111	
    LD PAOR, A  
    
				; inicjalizacja zmiennych
		LD		A, #0
		LD		maxtemp, A
		LD		A, #99
		LD		mintemp, A	
		
		RSP					; ustawienie domylnej wartoci wskanika stosu
		
		CALL	lcdinit	; inicjalizacja LCD
		WTXT	txt
		LD 		A, #$A8
		CALL	writecommand	; przejcie do drugiej linii wywietlacza
		WTXT		txt1
		; zapis do pamieci CG RAM symbolu "stopie"
		LD		A, #64
		CALL	writecommand
		WTXT stopien
		; / zapis do pamici generatora znakw symbolu "stopie"
		


		CALL	gettemp	; pierwszy pomiar temperatury w celu odrzucenia wyniku 85 stopni

;**************************************************************************************************
; gwna ptla programu	
;**************************************************************************************************
mainloop
		CALL	gettemp 			; odczyt temperatury z czytnika
		CALL	getminmax			; obliczenie minimalnej i maksymalnej temperaury
		CALL	displaytemp		; wywietlenie temperatur na wywietlaczu
		JP		mainloop 			; skok do pocztku ptli gwnej

;**************************************************************************************************
; Procedura pomiaru temperatury
; Temperatura zostaje umieszczona w zmiennej temper
; oraz pozostaje w rejestrze A
;**************************************************************************************************
.gettemp
		CALL 	owreset				; reset magistrali 1wire
		JRC		gettemp				; jeli brak impulsu obecnoci to skok do gettemp
		LD 		A, #$CC		
		CALL 	owwrite				; zapis komendy SKIP ROM
		LD		A, #$44
		CALL	owwrite				; zapis komendy CONVERT
		
		CALL	waitEndConversion	;

		CALL 	owreset 			; reset magistrali 1wire
		LD 		A, #$CC		
		CALL 	owwrite				; zapis komendy SKIP ROM
		LD 		A, #$BE
		CALL 	owwrite 			; zapis komendy READ SCRATCHPAD
		CALL 	owread
		LD 		tmplsb, A 		; odczyt modszego bajtu wyniku pomiaru
		CALL 	owread
		LD 		tmpmsb, A			; odczyt starszego bajtu wyniku pomiaru

		LD		A, tmplsb
		SWAP	A							; modszy bajt dzielimy przez 16
		AND		A, #$0F				; zerujemy starsz powk bajtu
		SWAP 	tmpmsb				; starszy bajt mnozymy przez 16
		ADD		A, tmpmsb			; dodajemy uprzednio przetworzone bajty do siebie
		LD		temper, A 		; wynik zapisujemy do zmiennej temper
		RET
		
;**************************************************************************************************
; Procedura obliczajca minimaln i maksymaln temperature
; Temperatura wejciowa w zmiennej temper
; Dane wyjciowe w zmiennych mintemp i maxtemp
;**************************************************************************************************
.getminmax
		LD		X, temper			; do X adujemy aktualn temperatur
		CP		X, mintemp 		; porwnujemy z wartoci temperetury minimalnej	
		JRPL	gmm1 					; jeli temper > mintemp, skok do etyiety lminta
		LD		mintemp, X 		; w przeciwnym razie uaktualniamy mintemp
gmm1
		LD		X, temper			; do X adujemy aktualn temperatur
		CP		X, maxtemp 		; porwnujemy z wartoci temperatury maksymalnej
		JRMI	gmm2 					; jeli temper < maxtemp, skok do etykiety lmaxta
		LD		maxtemp, X 		; w przeciwnym razie uaktualniamy maxtemp
gmm2
		RET
		
;**************************************************************************************************
; Procedura wyswietlajca temperature na wyswieltaczu
; dane wejsciowe w zmiennych temper, mintemp i maxtemp
;
;**************************************************************************************************
displaytemp
		; wywietlenie wyniku pomiaru temperatury na wyswietlaczu LCD
		LD		A, #2
		CALL	writecommand 	; LCD HOME
		
		WTXT	txt2					
		
		LD		A, temper			
		CALL	writedecimal	; wywietlenie aktualnej temperatury
		LD		A, #0
		CALL	writedata
		LD		A, #'C'
		CALL	writedata
		LD		A, #' '
		CALL	writedata
		
		LD 		A, #$A8
		CALL	writecommand	; przejcie do drugiej linii wywietlacza
		
		WTXT	txt3
		
		LD		A, mintemp
		CALL 	writedecimal	; wywietlenie temperatury minimalnej
		
		WTXT	txt4
		
		LD		A, maxtemp
		CALL	writedecimal	; wywietlenie temperatury maksymalnej
		LD		A, #0
		CALL	writedata
		LD		A, #' '
		CALL	writedata
		; / wywietlenie wyniku pomiaru temperatury na wyswietlaczu LCD
		RET			
		
;******************************************************************************
; Koniec pliku main.asm
;******************************************************************************		
    END

