st7/
;******************************************************************************
; LITEcomp - aplikacje
; Biblioteka procedur obsugi wywietlacza NOKIA 3310 
; Plik : n3310lcd.asm
; Autor : Radosaw Kwiecie, radoslaw.kwiecien@ep.com.pl
;******************************************************************************

		.NOLIST	
		#INCLUDE "st7flite19.inc"
    .LIST 
	
		EXTERN 	font5x8
	
N3310_RES	EQU 0
N3310_DC	EQU	5

N3310C_FS							EQU	$20
N3310C_FS_BASIC				EQU	$00
N3310C_FS_EXTENDED		EQU	$01
N3310C_FS_HORIZONTAL	EQU $00
N3310C_FS_VERTICAL		EQU $02
N3310C_FS_PD					EQU	$04

N3310C_DC							EQU	$08
N3310C_DC_BLANK				EQU $00
N3310C_DC_NORMAL			EQU $04
N3310C_DC_ALLON				EQU $01
N3310C_DC_INVERT			EQU $05

N3310C_XADDR					EQU	$80
N3310C_YADDR					EQU $40

N3310C_TEMP						EQU	$04
N3310C_BIAS						EQU $10
N3310C_VOP						EQU	$40


;******************************************************************************
; obszar pamici danych RAM
;******************************************************************************
		BYTES 					
		SEGMENT 'ram0'
fontPtr	DS.W	1

;******************************************************************************
; obszar pamici programu
;******************************************************************************
		WORDS 					
		SEGMENT 'rom' 	

;******************************************************************************
; Procedura zapisujca do wywietlacza rozkaz
;******************************************************************************
.N3310_WriteCommand
		BREs	PBDR, #N3310_DC
		CALL	sendSPI
		RET
		
;******************************************************************************
; Procedura zapisujca do wywietlacza dan
;******************************************************************************
.N3310_WriteData
		BSET	PBDR, #N3310_DC
		CALL	sendSPI
		RET
		
;******************************************************************************
; Procedura generujca ujemny impuls na linii Reset
;******************************************************************************
.N3310_Reset
		BRES	PBDR, #N3310_RES
		NOP
		NOP
		BSET	PBDR, #N3310_RES
		RET

;******************************************************************************
; Procedura ustawiajca wsprzdne wywietlacza
;******************************************************************************
.N3310_GoTo
		LD		A, X
		ADD		A, #N3310C_XADDR
		CALL	N3310_WriteCommand
		LD		A, Y
		ADD		A, #N3310C_YADDR
		CALL	N3310_WriteCommand
		RET

;******************************************************************************
; Procedura inicjalizacji wywietlacza
;******************************************************************************
.N3310_Init
		BSET	PBDDR, #N3310_DC
		BSET	PBOR, #N3310_DC
		BSET	PBDDR, #N3310_RES
		BSET	PBOR, #N3310_RES
		BSET	PBDR, #N3310_RES
		
		CALL	initSPI
				
		CALL	N3310_Reset
		
		LD		A, #N3310C_FS
		OR		A, #N3310C_FS_EXTENDED
		CALL	N3310_WriteCommand
		
		LD		A, #N3310C_VOP
		OR		A, #72
		CALL	N3310_WriteCommand
		
		LD		A, #N3310C_TEMP
		OR		A, #2
		CALL	N3310_WriteCommand
		
		LD		A, #N3310C_BIAS
		OR		A, #3
		CALL	N3310_WriteCommand
		
		LD		A, #N3310C_FS
		OR		A, #N3310C_FS_BASIC
		OR		A, #N3310C_FS_HORIZONTAL
		CALL	N3310_WriteCommand
		
		LD		A, #N3310C_DC
		OR		A, #N3310C_DC_NORMAL
		CALL	N3310_WriteCommand
	
		CALL 	N3310_ClearScreen
		RET


;******************************************************************************
; Procedura wywietlajca bitmap
;******************************************************************************
.N3310_Bitmap
		LD		{fontPtr+0}, X			; przepisanie adresu bitmapy do wskanika
		LD		{fontPtr+1}, Y			; 
		
		CLR		X										;
		CLR		Y										;
		CALL	N3310_GoTo					; ustawienie wsprzdnych ekranu X = 0 Y = 0
		
		LD		Y, #6								; ilo wierszy
BitmapLoop1		
		LD		X, #84							; ilo kolumn
BitmapLoop2		
		LD		A,[fontPtr.w]				; odczytanie bajtu z tablicy
		CALL	N3310_WriteData			; wywietlenie na wyswietlaczu
		
		LD		A, {fontPtr+1}			;
		ADD		A, #1								; inkrementacja modszego bajtu wskanika
		LD		{fontPtr+1}, A			;
		
		LD		A, {fontPtr+0}			;		
		ADC		A, #0								; inkrementacja starszego bajtu wskanika
		LD		{fontPtr+0}, A			;
		
		DEC		X										; czy to ostatnia kolumna?
		JRNE	BitmapLoop2					;	jeli nie to skok do BitmapLoop2
		
		DEC		Y										; czy to ostatni wiersz?
		JRNE	BitmapLoop1					;	jeli nie to skok do BitmapLoop1
		RET
		

;******************************************************************************
; Procedura czyszczca pamie obrazu
;******************************************************************************
.N3310_ClearScreen
		CLR		X										;
		CLR		Y										; ustawienie wsprzdnych ekranu X = 0 Y = 0
		CALL	N3310_GoTo					; 
		
		LD		Y, #6								; ilo wierszy
ClearScreenLoop1		
		LD		X, #84							; ilo kolumn
ClearScreenLoop2		
		LD		A, #0								;			
		CALL	N3310_WriteData			; zapis wartoci czyszczcej ekran
		
		DEC		X										; czy to ostatnia kolumna?
		JRNE	ClearScreenLoop2		; jeli nie to skok do ClearScreenLoop2
		
		DEC		Y										; czy to ostatni wiersz?
		JRNE	ClearScreenLoop1		; jeli nie to skok do ClearScreenLoop1
		RET
		
;******************************************************************************
; Procedura wywietlajca na wywietlaczu znak o kodzie ASCII przekazanym w A
;******************************************************************************
.N3310_WriteChar
		PUSH	X										; zapamietanie X na stosie
		SUB		A, #32							; konwersja kodu
		
		LD		X, A								;
		LD		A, #5								; obliczenie przesunicia (5* kod znaku)
		MUL		X, A								; 
		
		LD		{fontPtr+0},X				; przepisanie wyniku mnoenia do fontPtr
		LD		{fontPtr+1},A				;
		
		LD		A, {fontPtr+1}			; (LSB)
		ADD		A, #font5x8.l				; Zsumowanie obliczonego przesunicia z 
		LD	  {fontPtr+1}, A			; adresem pocztku tablicy czcionek 
		
		LD		A, {fontPtr+0}			; (MSB)
		ADC		A, #font5x8.h				;
		LD	  {fontPtr+0}, A			;

		CLR		X
WriteCharLoop
		LD		A, ([fontPtr.w],X)	; odczyt danych z tablicy
		CALL	N3310_WriteData			; wywietlenie na wywietlaczu
		INC		X
		CP		X, #5								; czy to ju cay znak?
		JRNE	WriteCharLoop				; jeli nie to skok do WriteCharLoop
		
		LD		A, #0								;
		CALL	N3310_WriteData			; odstp pomidzy znakami
		
		POP		X
		RET

;******************************************************************************
; Procedura inicjalizacji interfejsu SPI
;******************************************************************************
initSPI
		LD		A, #%00000011 		; programowe zarzdzanie trybem pracy, SS -> GPIO
		LD		SPISR, A
		LD		A, #%01010000 		; MASTER; Enable, Fsck = Fcpu/8
		LD		SPICR, A
		RET
		
;******************************************************************************
; Procedura  transmitujca  bajt danych przez interfejs SPI
;******************************************************************************
sendSPI
		LD		SPIDR, A					; zaadowanie danej do wysania do SPIDR
		BTJF	SPISR, #7, *			; oczekiwanie na zakoczenie transmisji
		LD		A, SPIDR					; zaadowanie odebranej danej do A
		RET
		
		END
;******************************************************************************
; Koniec pliku n3310lcd.asm
;******************************************************************************