//--------------------------------------------------------------------------------------------------
// LITEcomp - aplikacje
// Biblioteka funkcji obsugi wywietlacza LCD w jzyku C
// Plik : lcd.c
//--------------------------------------------------------------------------------------------------
#ifdef __RAISONANCE__
	#include <st7lite1.h>
	#include <intrins.h>
#endif

#ifdef __CSMC__
	#include <io7flit1.h>
	#define _nop_() _asm("nop")
#endif

#include "lcd.h"

//--------------------------------------------------------------------------------------------------
// Funkcja opniajca o 10us dla czstotliwoci 16MHz
//--------------------------------------------------------------------------------------------------
void wait_10us(void)
{
unsigned char i;
_nop_();
for(i = 0; i < 10; i++);
}
//--------------------------------------------------------------------------------------------------
// Funkcja opniajca o ok. delay*10us dla czstotliwoci 16MHz
//--------------------------------------------------------------------------------------------------
void LCD_Delay(unsigned char delay)
{
for( ;delay > 0; delay--)
  wait_10us();
}
//--------------------------------------------------------------------------------------------------
// Funkcja wystawiajca na port 4-bity danych
//--------------------------------------------------------------------------------------------------
void LCD_OutNibble(unsigned char nibble)
{
LCD_PORT &= 0x0F;
LCD_PORT |= (nibble << 4);
}
//--------------------------------------------------------------------------------------------------
// Funkcja zapisujca do wywietlacza bajt (bez rozrnienia dane/instrukcja)
//--------------------------------------------------------------------------------------------------
void LCD_Write(unsigned char dataToOut)
{

LCD_OutNibble(dataToOut >> 4); // zapis starszego pbajtu
LCD_PORT |= LCD_EN;
LCD_PORT &= ~LCD_EN;

LCD_OutNibble(dataToOut & 0x0F); // zapis modszego pbajtu
LCD_PORT |= LCD_EN;
LCD_PORT &= ~LCD_EN;
}

//--------------------------------------------------------------------------------------------------
// Funkcja zapisujca do wywietlacza instrukcj
//--------------------------------------------------------------------------------------------------
void LCD_WriteCommand(unsigned char commandToWrite)
{
LCD_PORT &= ~LCD_RS; // zapis instrukcji
LCD_Write(commandToWrite); // zapis bajtu
LCD_Delay((commandToWrite>2)?5:165); // jesli instrukcja CLEAR lub HOME to czekaj ok 1,65ms
												 // w przeciwnym razie czekaj 50us.
}

//--------------------------------------------------------------------------------------------------
// Funkcja zapisujaca do wywietlacza dan
//--------------------------------------------------------------------------------------------------

void LCD_WriteData(unsigned char dataToWrite)
{
LCD_PORT |= LCD_RS;
LCD_Write(dataToWrite);
LCD_Delay(5);
}

//--------------------------------------------------------------------------------------------------
// Funkcja zapisujaca do wywietlacza tekst
//--------------------------------------------------------------------------------------------------
void LCD_WriteText(char * text)
{
while(*text)
  LCD_WriteData(*text++);
}

//--------------------------------------------------------------------------------------------------
// Funkcja ustawiajca wsprzdne na wywietlaczu
//--------------------------------------------------------------------------------------------------
void LCD_GoTo(unsigned char x, unsigned char y)
{
LCD_WriteCommand(HD44780_DDRAM_SET + x + (64 * y));
}

//--------------------------------------------------------------------------------------------------
// Funkcja inicjujca wywietlacz.
//--------------------------------------------------------------------------------------------------
void LCD_Init(void)
{
unsigned char i;
PADDR |= 0b11111010; // inicjalizacja portw
PAOR |= 0b11111010;

LCD_Delay(50); // oczekiwanie na ustabilizowanie napicia zasilania

for(i = 0; i < 3; i++)
  {
  
  LCD_OutNibble(0x03); 
  LCD_PORT |= LCD_EN;
	LCD_PORT &= ~LCD_EN;
  LCD_Delay(250);
  LCD_Delay(250);
  }


LCD_OutNibble(0x02);
LCD_PORT |= LCD_EN;
LCD_PORT &= ~LCD_EN;
LCD_Delay(100);

LCD_WriteCommand(HD44780_FUNCTION_SET | HD44780_FONT5x7 | HD44780_TWO_LINE | HD44780_4_BIT);
LCD_WriteCommand(HD44780_DISPLAY_ONOFF | HD44780_DISPLAY_OFF); 
LCD_WriteCommand(HD44780_CLEAR); 
LCD_WriteCommand(HD44780_ENTRY_MODE | HD44780_EM_SHIFT_CURSOR | HD44780_EM_INCREMENT);
LCD_WriteCommand(HD44780_DISPLAY_ONOFF | HD44780_DISPLAY_ON | HD44780_CURSOR_OFF | HD44780_CURSOR_NOBLINK); 
}
//--------------------------------------------------------------------------------------------------
// Koniec pliku lcd.c
//--------------------------------------------------------------------------------------------------




